require_relative 'messages'

module RkCucumber
  class Config
    class << self

      def default
        {
          project_name: 'Default',
          report_server_host: 'results-keeper.com',
          report_server_port: 80,
          api: {
            revision_path: 'revisions',
            test_path: 'tests'
            }
        }
      end

      def config
        opts = {}
        opts[:report_server_host] = ENV['RK_REPORT_SERVER_HOST'] if ENV['RK_REPORT_SERVER_HOST']
        opts[:report_server_port] = ENV['RK_REPORT_SERVER_PORT'] if ENV['RK_REPORT_SERVER_PORT']
        opts[:project_name] = ENV['RK_PROJECT_NAME'] || default_project_name
        opts[:revision_name] = ENV['RK_REVISION_NAME'] || "Result for #{Time.now}"
        opts[:secret_key] = ENV['RK_SECRET_KEY'] || raise(RkCucumber::Messages.no_secret_key)
        default.merge!(opts)
      end

      private

      def default_project_name
        begin
          `pwd`.strip.split('/').last
        rescue
          'Default'
        end
      end
    end
  end
end