require 'net/http'
require 'json'
require 'rest_client'
require_relative 'messages'
require_relative 'test_data_generator'
require_relative 'config'
require 'net/http/persistent'

module RkCucumber
  class ResultsSender

    attr_reader :tmp_data

    def initialize
      @data_generator = RkCucumber::TestDataGenerator.new
      @tmp_data ||= {}
      @tmp_data[:tests] ||= []
    end

    def send_revision
      @revision = send_json(@data_generator.revision_data,
                            RkCucumber::Config.default[:api][:revision_path],
                            "#{RkCucumber::Config.config[:project_name]} > #{RkCucumber::Config.config[:revision_name]} started")
    end

    def send_revision_complete
      send_not_sent_scenarios
      send_json(@data_generator.revision_completed_data,
                RkCucumber::Config.default[:api][:revision_path],
                "#{RkCucumber::Config.config[:revision_name]} completed")
    end

    def send_test(scenario, test_started_at)
      # In case if the host is not available we need to send revision once more (when it comes back)
      send_revision unless @revision
      return nil unless @revision
      screenshot_path = @data_generator.save_screenshot(scenario) if ENV['RK_SEND_SCREENSHOTS']
      @test = send_json(@data_generator.test_data(scenario, @revision, test_started_at),
                        RkCucumber::Config.default[:api][:test_path])
      return nil unless @test
      send_screenshot(screenshot_path) if screenshot_path
      @test
    end

    def send_not_sent_scenarios
      if @revision
        @tmp_data[:tests].each do |s|
          s[:revision_id] = @revision['revision_id'] unless s[:revision_id]
          RkCucumber::Messages.info_trying_resend(s[:name])
          send_json(s, RkCucumber::Config.default[:api][:test_path]) if @revision
        end
      else
        false
      end
    end

    def send_json(body, path, message_to_client=nil)
      body['secret_key'] = RkCucumber::Config.config[:secret_key]
      @path = "/api/#{path}"
      @body = body.to_json
      begin
        uri = URI("#{RkCucumber::Config.config[:report_server_host]}:#{RkCucumber::Config.config[:report_server_port]}#{@path}")
        http = Net::HTTP::Persistent.new
        http.verify_mode = OpenSSL::SSL::VERIFY_NONE
        request = Net::HTTP::Post.new(@path, initheader = {'Content-Type' => 'application/json'})
        request.body = @body
        response = http.request(uri, request)
        RkCucumber::Messages.info_message_to_client(response, message_to_client)
        JSON.parse(response.body)
      rescue
        RkCucumber::Messages.warn_problem_with_rk
        false
      end
    end

    def send_screenshot(screenshot_path)
      begin
        params = {project: @revision['project_id'], revision: @revision['revision_id'], test: @test['id']}
        RestClient::Request.execute(:url => "#{RkCucumber::Config.config[:report_server_host]}:"+
            "#{RkCucumber::Config.config[:report_server_port]}/api/tests/upload_screenshot",
                                    :method => :post,
                                    :verify_ssl => false,
                                    :name_of_file_param => File.new(screenshot_path),
                                    :payload => {:multipart => true, :name_of_file_param => File.new(screenshot_path)},
                                    :headers => {:accept => :json, params: {body: params}})
        File.delete(screenshot_path)
      rescue
        RkCucumber::Messages.error_send_screenshot
      end
    end

    def add_test_to_tmp(scenario, test_started_at)
      @tmp_data[:tests] << @data_generator.test_data(scenario, @revision, test_started_at)
      RkCucumber::Messages.error_not_sent_scenario(@tmp_data[:tests].count)
    end
  end
end