require_relative 'config'
require_relative 'messages'

module RkCucumber
  class TestDataGenerator

    SCENARIO_HOOKS = 'Before hook  After hook  Around hook'

    def revision_data
      { revision: RkCucumber::Config.config[:revision_name], project: RkCucumber::Config.config[:project_name] }
    end

    def revision_completed_data
      { revision: RkCucumber::Config.config[:revision_name], project: RkCucumber::Config.config[:project_name], completed: 1 }
    end

    def test_data(scenario, revision, test_started_at)
      revision ||= { revision_id: nil }
      test_duration = Time.now - test_started_at
      scenario_error = scenario.exception.message if scenario.exception
      run_path = "cucumber #{scenario.location.file}:#{scenario.location.line}"
      {
        name: scenario.name,
        status: scenario.status,
        feature_name: scenario.feature.name,
        run_path: run_path,
        tags: scenario.tags.map(&:name).join(', '),
        error: scenario_error,
        revision_id: revision['revision_id'],
        steps: scenario_steps(scenario),
        duration: test_duration
      }
    end

    def save_screenshot(scenario)
      if scenario.exception
        begin
          screenshot_name = "#{Time.now.to_i}_#{rand(1000..9999)}.png"
          @file_path = "tmp/screenshots/#{screenshot_name}"
          Capybara.page.save_screenshot(@file_path)
          @file_path
        rescue
          RkCucumber::Messages.take_screenshot_error
          nil
        end
      end
    end

    private

    def step_keyword(test_step)
      test_step.source.last.keyword
    end

    def scenario_steps(scenario)
      steps = []
      scenario.test_steps.each do |test_step|
        if test_step.to_s.include? 'Cucumber::Core::Test'
          steps << "#{step_keyword(test_step)} #{test_step.name.to_s}" unless SCENARIO_HOOKS.include?(test_step.name.to_s)
        else
          steps << "#{step_keyword(test_step)} #{test_step.to_s}" unless SCENARIO_HOOKS.include?(test_step.to_s)
        end
      end
      steps
    end
  end
end